library bloc_bus;

import 'dart:async';

import 'package:bloc/bloc.dart';
import 'package:event_bus/event_bus.dart';
import 'package:flutter/widgets.dart';

extension BlocBus<E, S> on Bloc<E, S> {
  // ignore: avoid_shadowing_type_parameters
  StreamSubscription<E> onBus<E>(void Function(E) subscription) =>
      eventBus.on<E>().listen(subscription);
}

EventBus eventBus = EventBus();

typedef OnError = void Function(
    BlocBase? bloc, Object error, StackTrace? stackTrace)?;

R runZonedBlocBus<R>(
  R Function() body, {
  Stream<dynamic> Function(Stream<dynamic>, Stream<dynamic> Function(dynamic))?
      eventTransformer,
  OnError onError,
}) {
  FlutterError.onError = (FlutterErrorDetails details) {
    onError?.call(null, details.exception, details.stack);
  };
  return BlocOverrides.runZoned<R>(
    body,
    blocObserver: BlocBusObserver(onError),
    eventTransformer: eventTransformer,
  );
}

class BlocBusObserver extends BlocObserver {
  final OnError onBlocError;

  BlocBusObserver(this.onBlocError);

  @override
  void onEvent(Bloc bloc, Object? event) {
    super.onEvent(bloc, event);
    eventBus.fire(event);
  }

  @override
  void onChange(BlocBase bloc, Change change) {
    super.onChange(bloc, change);
    eventBus.fire(change.nextState);
  }

  @override
  // ignore: unnecessary_overrides
  void onTransition(Bloc bloc, Transition transition) {
    super.onTransition(bloc, transition);
    // eventBus.fire(transition.event);
  }

  @override
  // ignore: unnecessary_overrides
  void onError(BlocBase bloc, Object error, StackTrace stackTrace) {
    super.onError(bloc, error, stackTrace);
    onBlocError?.call(bloc, error, stackTrace);
  }
}
